#
# This file is part of GNU Enterprise.
#
# GNU Enterprise is free software; you can redistribute it
# and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either
# version 2, or (at your option) any later version.
#
# GNU Enterprise is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied
# warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
# PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public
# License along with program; see the file COPYING. If not,
# write to the Free Software Foundation, Inc., 59 Temple Place
# - Suite 330, Boston, MA 02111-1307, USA.
#
# Copyright 2001-2002 Free Software Foundation
#
# FILE:
# MenuBar.py
#
# DESCRIPTION:
#
# NOTES:

import sys, os, string
from wxPython.wx import *
from gnue.common import GDebug
import MRUManager
from ToolSupport import SupportedTools

#
#
#
class MainMenuBar(wxMenuBar):
  def __init__(self, frame):
     wxMenuBar.__init__(self)

     # Keep track of user-installed + sys-installed wizards/tools
     self._menuMappings = {}
     self._eventMappings = {}

     # So we can honor the module's requested order for the tool menu
     self._toolMenuOrder = []

     self._frame = frame

  def addWizardLocation(self, location, template):
    #print "Adding %s" % location
    wr = self._frame.wizardRunner(template,
                                  string.split(location,'|')[-1],
                                  self._frame)

    self._menuMappings[location] = wr

  # Called after all the addWizardLocation's have been run
  def buildMenu(self):
     frame = self._frame
     # File menu
     ID_NEW = wxNewId()
     ID_OPEN = wxNewId()
     ID_OPEN_RECENT = wxNewId()
     ID_SAVE = wxNewId()
     ID_SAVE_AS = wxNewId()
     ID_SAVE_ALL = wxNewId()
     ID_CONNECT = wxNewId()
     ID_CLOSE = wxNewId()
     ID_RELOAD = wxNewId()
     ID_EXIT = wxNewId()

     # FileNew menu
     ID_NEW_FORM = wxNewId()
     ID_NEW_REPORT = wxNewId()
     ID_NEW_WIZARD = wxNewId()

     # Edit menu
     ID_CUT = wxNewId()
     ID_COPY = wxNewId()
     ID_PASTE = wxNewId()
     ID_PASTE_SPECIAL = wxNewId()
     ID_DELETE = wxNewId()

     # Project menu
     ID_NEW_PROJECT = wxNewId()
     ID_OPEN_PROJECT = wxNewId()

     # Debug menu
     ID_RUN = wxNewId()

     # Help menu
     ID_ABOUT = wxNewId()



     self._file = wxMenu()
     self._fileNew = wxMenu()
     self._fileRecent = wxMenu()
     self._fileConnect = wxMenu()

     self._file.AppendMenu(ID_NEW, _("&New"), self._fileNew, _("Create a new object"))
     self._file.Append(ID_OPEN, _("&Open..."), _("Open an existing object"))
     self._file.AppendMenu(ID_OPEN_RECENT, _("Open &Recent"), self._fileRecent,
                           _("Open an existing object"))
     self._file.AppendSeparator()
     self._file.Append(ID_SAVE, _("&Save"), _("Save the current object"))
     self._file.Append(ID_SAVE_AS, _("Save &As..."),
                       _("Save the current form under a new name"))
     self._file.Append(ID_SAVE_ALL, _("Save A&ll"),
                       _("Save all open objects"))
     self._file.AppendSeparator()
     self._file.AppendMenu(ID_CONNECT, _("Connect &to"), self._fileConnect,
                           _("Connect to a database"))
     self._file.AppendSeparator()
     self._file.Append(ID_RELOAD, _("Reload"),
        _("Reload the current object as of its last save (abandoning any changes)"))
     self._file.AppendSeparator()
     self._file.Append(ID_CLOSE, _("&Close"), _("Close the current object"))
     self._file.AppendSeparator()
     self._file.Append(ID_EXIT, _("E&xit"), _("Exit GNUe Designer"))


     self.__addMenuEvent('File|New|Wizard', ID_NEW_WIZARD, frame._app.OnWizard)
     self.__addMenuEvent('File|Open', ID_OPEN, frame._app.OnOpen)
     self.__addMenuEvent('File|Save', ID_SAVE, frame.OnSave)
     self.__addMenuEvent('File|Save As', ID_SAVE_AS, frame.OnSaveAs)
     self.__addMenuEvent('File|Save All', ID_SAVE_ALL, frame._app.OnSaveAll)
     self.__addMenuEvent('File|Close', ID_CLOSE, frame.OnClose)
     self.__addMenuEvent('File|Exit', ID_EXIT, frame._app.OnExit)

     # Set up our MRU list
     frame._app.mru.addMenu(self._fileRecent, frame)


     frame._fileNewMap = {}
     for tool in SupportedTools:
       id = wxNewId()
       frame._fileNewMap[id] = tool.properties.module
       self._fileNew.Append(id, '&%s' % tool.properties.nickname,
         _('Create a new %s') % string.lower(tool.properties.nickname))
       self.__addMenuEvent('File|New|%s' % tool.properties.nickname, id, frame.OnNew)


     self._fileNew.AppendSeparator()
     self._fileNew.Append(ID_NEW_WIZARD,_('from &Wizard...'), \
                          _("Create a new object from a wizard"))

     # Build a connection list
     frame.connectionMap = {}
     for conn in frame.connections.getAllConnectionParameters().keys():
       id = wxNewId()
       self._fileConnect.Append(id, conn, _("Login to %s connection") % conn)
       frame.connectionMap[id] = conn
       self.__addMenuEvent('File|Connect To|%s' % conn, id, frame.OnConnectTo)


     self._file.Enable(ID_RELOAD,0)

     self._help = wxMenu()
     self._help.Append(ID_ABOUT, _("&About..."), _("More information about GNUe Designer"))
     self.__addMenuEvent('Help|About', ID_ABOUT, frame._app.OnAbout)

     self._edit = wxMenu()
     self._edit.Append(ID_CUT, _("C&ut"), _("Cut the current object and move to the clipboard"))
     self._edit.Append(ID_COPY, _("&Copy"), _("Copy the current object to the clipboard"))
     self._edit.Append(ID_PASTE, _("&Paste"), _("Paste the current object on the clipboard"))
     self._edit.Append(ID_PASTE_SPECIAL, _("Paste &Special..."), _("Paste the current object on the clipboard with special attributes"))
     self._edit.AppendSeparator()
     self._edit.Append(ID_DELETE, _("&Delete"), _("Delete the current object"))

     # TODO: Implement clipboard
     self._edit.Enable(ID_CUT,0)
     self._edit.Enable(ID_COPY,0)
     self._edit.Enable(ID_PASTE,0)
     self._edit.Enable(ID_PASTE_SPECIAL,0)
     self._edit.Enable(ID_DELETE,0)


     # TODO: Implement projects
     self._project = wxMenu()


     self._tools = wxMenu()
     for location in self._toolMenuOrder:

       # Grab our ToolRunner (or WizardRunner if tool was replaced???)
       # and delete the reference.
       tool = self._menuMappings[location]
       del self._menuMappings[location]

       menu = wxMenuItem(self._tools, tool.wxId, tool.title,
                         tool.tooltip, tool.checkable)
       tool.menu = menu
       self._tools.AppendItem(menu)
       self.__addMenuEvent('%s|%s' % (location, tool.title),
                           tool.wxId, tool.run)
       tool.finalize()

     self._tools.AppendSeparator()


     self._debug = wxMenu()
     self._debug.Append(ID_RUN, _('&Run'), _("Run the current form"))
     self.__addMenuEvent('Debug|Run', ID_RUN, frame.OnDebugRun)

     menumap = { '': (self, {}),
                 'File': (self._file,{}),
                 'File|New': (self._fileNew,{}),
                 'Edit': (self._edit,{}),
                 'Project': (self._project,{}),
                 'Debug': (self._debug,{}),
                 'Tools': (self._tools,{}),
                 'Help': (self._help, {})}

     basemenuorder = ['File','Edit','Debug','Project', 'Tools']

     for location in self._menuMappings.keys():
       last = None
       nav = string.split(location,'|')
       partial = ""
       lastpartial = ""
       for loc in nav[:-1]:
         lastpartial = partial
         partial += (partial and '|' or "") + loc
         if not menumap.has_key(partial):
           menumap[partial] = (wxMenu(), {})
           menumap[lastpartial][1][loc] = (1, menumap[partial][0])

       menumap[partial][1][nav[-1]] = (0, location)


     # Add all the base menus first...
     for name in basemenuorder:
       self.Append(menumap[name][0], '&' + name)


     # Next, add all the customizations
     locsort = menumap.keys()
     locsort.sort()
     for location in locsort:
       menu, submenus = menumap[location]
       mm = submenus.keys()
       mm.sort()
       for menuitem in mm:
         isMenu, data  = submenus[menuitem]
         if isMenu:
           # data is a wxMenu instance
           menu.AppendMenu(wxNewId(), '&' + menuitem, data)
         else:
           # Data is a location pointer
           wizard = self._menuMappings[data]
           menu.Append(wizard.wxId, wizard.title, wizard.tooltip)
           self.__addMenuEvent('%s|%s' % (location, wizard.title),
                               wizard.wxId, wizard.run)


     # Add the Help menu
     self.Append(menumap['Help'][0], '&Help')


  def __createMenuStructure(location, data):
    for loc, isMenu, data in menumap[name][1]:
     if isMenu:
       menu.AppendMenu(wxNewId(), "&%s" % loc, data)
     else:
       pass


  def __addMenuEvent(self, location, wxId, method):
    self._eventMappings[location] = (method, wxId)
    EVT_MENU(self._frame, wxId, method)


  def getEventHelp (self, location):
    try:
      wxId = self._eventMappings[location][1]
      return self.GetHelpString(wxId)
    except KeyError:
      return None


  def simulateMenuEvent(self, location):
    method, id = self._eventMappings[location]
    method(wxEvent(id))



  def addTool(self, toolFrame, title):
    location = 'Tools|%s' % title
    self._menuMappings[location] = ToolRunner(toolFrame, title)
    self._toolMenuOrder.append(location)
    return self._menuMappings[location]


  #
  # Convenience methods to make the main menu bar appear to be a menu container
  #

  def AppendMenu(self, wxId, text, menu):
    self.Append(menu, text)



class ToolRunner:
  def __init__(self, toolFrame, title):
     self.toolFrame = toolFrame

     self.wxId = wxNewId()
     self.title = title
     self.checkable = 1
     self.tooltip = _("Show or hide the %s") % title

  def run(self, event):
    self.toolFrame.OnMenuSelected(event)

  def finalize(self):
    self.toolFrame.finalize()
