#
# Copyright 2001 Free Software Foundation
#
# This file is part of GNU Enterprise.
#
# GNU Enterprise is free software; you can redistribute it
# and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either
# version 2, or (at your option) any later version.
#
# GNU Enterprise is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied
# warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
# PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public
# License along with program; see the file COPYING. If not,
# write to the Free Software Foundation, Inc., 59 Temple Place
# - Suite 330, Boston, MA 02111-1307, USA.
#
# FILE:
# Instance.py
#
# DESCRIPTION:
#
# NOTES:
#

import os, sys
from gnue.common import RuntimeSettings
from wxPython.wx import *
from gnue.common import GDebug

#
# Any tool frames inherit from this
# It handles the Window menu and
# saving size/position/display settings
#
class ToolFrame (wxFrame):

  default_width = 200
  default_height = 150
  default_visible = 0

  def __init__(self, instance, title):
    if sys.platform[:3] == 'win':
      style = wxFRAME_TOOL_WINDOW|wxMINIMIZE_BOX|wxRESIZE_BORDER| \
              wxCAPTION|wxFRAME_FLOAT_ON_PARENT|wxSYSTEM_MENU
    else:
      style = wxMINIMIZE_BOX|wxRESIZE_BORDER|wxCAPTION|wxSYSTEM_MENU

    wxFrame.__init__(self, instance, -1, title, style=style)

    self.title = title
    self.runtime_section = "ToolFrame-%s" % string.replace(self.title,' ','')
    self.instance = instance

    # Add ourselves to the Window menu
    self.menutool = self.instance._menubar.addTool(self, title)

    EVT_CLOSE(self, self.OnClose)


  def finalize(self):

    self.SetPosition((
       RuntimeSettings.getint(self.runtime_section, 'x', -1),
       RuntimeSettings.getint(self.runtime_section, 'y', -1)))

    self.SetSize((
       RuntimeSettings.getint(
           self.runtime_section, 'width', self.default_width),
       RuntimeSettings.getint(
           self.runtime_section, 'height', self.default_height)))

    if RuntimeSettings.getint(
           self.runtime_section, 'visible', self.default_visible):
      self.show()

    RuntimeSettings.registerRuntimeSettingHandler(self.instance, self)


  def saveRuntimeSettings(self):
    x, y = self.GetPositionTuple()
    width, height = self.GetSizeTuple()
    return ( self.runtime_section,
             {'x': x,
              'y': y,
              'height': height,
              'width': width,
              'visible': self.menutool.menu.IsChecked() } )


  # This event is set by MenuBar.addTool
  # Called whenever menu item is selected
  def OnMenuSelected(self, event):
    if self.menutool.menu.IsChecked():
      self.show()
    else:
      self.hide()

  def show(self):
    self.menutool.menu.SetHelp(_("Hide the %s window")  % self.title)
    self.menutool.menu.Check(1)
    self.Show(1)

  def hide(self):
    self.menutool.menu.SetHelp(_("Show the %s window") % self.title)
    self.menutool.menu.Check(0)
    self.Show(0)

  def OnClose(self, event):
    self.hide()
    event.Veto()



