#
# This file is part of GNU Enterprise.
#
# GNU Enterprise is free software; you can redistribute it
# and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either
# version 2, or (at your option) any later version.
#
# GNU Enterprise is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied
# warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
# PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public
# License along with program; see the file COPYING. If not,
# write to the Free Software Foundation, Inc., 59 Temple Place
# - Suite 330, Boston, MA 02111-1307, USA.
#
# Copyright 2001-2002 Free Software Foundation
#
# FILE:
# TreeView.py
#
# DESCRIPTION:
#
# NOTES:
#
# TODO: * all the SetItemImage lines in this file have been commented
#         out as on win32 it was segfaulting
#       * make TreeView more stable, at the moment it fires OnTreeItemSelected
#         events while beeing destroyed, which is causing segfaults on wxPy<2.3
#         (wxPy 2.3 raises exceptions) when TreeView is not the first tool to
#         be removed

import sys, os
from wxPython.wx import *
from gnue.common import GDebug, GTrigger
from gnue.forms import GFForm, GFObjects, GFLibrary
from PopupMenu import ObjectMenu
from Icons import treeIconMap, treeIconList

class TreeView (wxTreeCtrl):
  def __init__(self, instance, form, parent):
    wxTreeCtrl.__init__(self, parent, wxNewId(),
       style=wxTR_HAS_BUTTONS|wxTR_EDIT_LABELS|wxTR_HAS_VARIABLE_ROW_HEIGHT)
    self.instance = instance
    self.rootObject = form
    self._handler = None

    self.SetImageList(treeIconList)

    EVT_TREE_BEGIN_LABEL_EDIT(self, self.GetId(), self.OnTreeLabelEdit)
    EVT_TREE_END_LABEL_EDIT(self, self.GetId(), self.OnTreeLabelEditEnd)
    EVT_TREE_SEL_CHANGED(self, self.GetId(), self.OnTreeItemSelected)

    self.rootObject._treeItem = self.AddRoot(self.rootDescr)

    EVT_RIGHT_DOWN(self, self.OnRightDown)
    EVT_LEFT_DCLICK(self, self.OnEditProperties)

  def AppendItem(self, parent, description):
    rv = wxTreeCtrl.AppendItem(self, parent, description)
    return rv

  def onSetCurrentObject (self, object, handler):
    if object == None:
      return
    if handler != __name__:
      self.EnsureVisible(object._treeItem)
      self._handler = handler
      self.SelectItem(object._treeItem)
      self._handler = None

  def onCreateObject (self, object, handler):
    if object == None:
      return
    if handler != __name__:
      try:
        GDebug.printMesg(4,"Adding %s (%s)" % (object.name, object._type))
      except: 
        pass
      self.inventoryObject(object)

  def onModifyObject (self, object, handler, modifications):
    if object == None:
      return
    if handler != __name__:
      for modification in modifications:
        attribute, value = modification
        if attribute == 'name' and object != self.rootObject:
          self.SetItemText(object._treeItem, object.getDescription())

  def onDeleteObject (self, object, handler):
    if object == None:
      return
    if hasattr(object, '_treeItemTrigger'):
      self.Delete(object._treeItemTrigger)
    if hasattr(object, '_treeItem'):
      if self.GetSelection() == object._treeItem:
        self.SelectItem(self.GetItemParent(object._treeItem))
      self.Delete(object._treeItem)
    self.Refresh()

  def OnTreeLabelEdit(self, event):
    event.Veto()

  def OnTreeLabelEditEnd(self, event):
    pass

  def OnTreeItemSelected(self, event):
    if self._handler == None:
      object = self.GetPyData(event.GetItem())
      if object != None and not isinstance(object, ObjectMenu):
        self.instance.onSetCurrentObject(object, __name__)
      else:
        self.instance.onSetCurrentObject(None, __name__)


  def OnRightDown(self, event):
    id, flags = self.HitTest(event.GetPosition())
    object = self.GetPyData(id)
    self.EnsureVisible(id)
    self.SelectItem(id)
    if isinstance(object, ObjectMenu):
      self.PopupMenu(object, event.GetPosition())
    elif object != None:
      self.PopupMenu(object._popupMenu, event.GetPosition())
      

  def OnEditProperties(self, event): 

    id, flags = self.HitTest(event.GetPosition())
    object = self.GetPyData(id)
    self.EnsureVisible(id)
    self.SelectItem(id)
    if not isinstance(object, ObjectMenu): 
      self.instance.propertyEditorWindow.Show(1)
      self.instance.propertyEditorWindow.Raise()
      self.instance.propertyEditorWindow.SetFocus()


