#
# This file is part of GNU Enterprise.
#
# GNU Enterprise is free software; you can redistribute it
# and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either
# version 2, or (at your option) any later version.
#
# GNU Enterprise is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied
# warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
# PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public
# License along with program; see the file COPYING. If not,
# write to the Free Software Foundation, Inc., 59 Temple Place
# - Suite 330, Boston, MA 02111-1307, USA.
#
# Copyright 2001-2002 Free Software Foundation
#
# FILE:
# form/Instance.py
#
# DESCRIPTION:
# Creates a "Form" instance
#
# NOTES:
#

from gnue.designer.forms import properties as formProperties
from gnue.designer.Instance import BaseInstance
from gnue.designer import PopupMenu
import Incubator
import wizards
from WizardRunner import WizardRunner

# Tool support...
from LayoutEditor import LayoutEditor
from TreeView import TreeView
from gnue.designer.PropertyEditor import PropertyEditor
from gnue.designer.TriggerEditor import TriggerEditor
from gnue.designer.DataSourceEditor import DataSourceEditor
from gnue.designer.SchemaViewer import SchemaViewer

from gnue.designer.PopupMenu import ObjectMenu

from gnue.forms import GFInstance
from gnue.forms import GFForm, GFParser, GFObjects, GFLibrary
from gnue.common import GTrigger

class Instance(BaseInstance, GFInstance.GFInstance):

  def __init__(self, app, *args, **params):

    self.incubator = Incubator
    self.properties = formProperties
    self.wizardRunner = WizardRunner

    # TODO: Can we make it so the GFInstance is not
    # TODO: part of the main Instance class, but an
    # TODO: attribute such as Instance.formInstance?

    GFInstance.GFInstance.__init__(self, app, -1,
                                   app.connections,
                                   None, disableSplash=1)

    # For GFInstance support, iirc
    self._pages = []

    BaseInstance.__init__(self, app, *args, **params)


  def loadBuffer(self, buffer):
    form = GFParser.loadFile (buffer, self, initialize=0)
    form.name = 'form'
    return form


  def loadEmpty(self):
    form = GFForm.GFForm()
    options = GFObjects.GFOptions(form)
    form.title = _("Untitled Form")
    form.height = 12
    form.name = 'form'
    form.width = 40
    GFObjects.GFPage(form)
    return form


  def inventoryObject(self, object):
    if isinstance(object, GFObjects.GFPage) and \
        object._parent == self.rootObject:
      self._pages.append(object)

    object._popupMenu = ObjectMenu(self, object)


  def createVisualEditor(self):
    return LayoutEditor(self, self)


  def createTools(self):
    self.addTool('propertyEditor', _('Property Inspector'), PropertyEditor)
    self.addTool('treeEditor', _('Object Navigator'), TreeView)
    self.addTool('triggerEditor', _('Event Editor'), TriggerEditor)
    self.addTool('datasourceEditor', _('Data Source Editor'), DataSourceEditor)
    self.addTool('schemaViewer', _('Schema Navigator'), SchemaViewer)

    self.loadWizards(wizards)


##  def insertDataSourceTool(self, event=None, params=None):
##    pass
##
##  def insertPageTool(self, event=None, params=None):
##    pass
##
##  def insertBlockTool(self, event=None, params=None):
##    pass
##
##  def insertFieldTool(self, event=None, params=None):
##    pass
##
##  def insertDropdownTool(self, event=None, params=None):
##    pass

  def preSave(self):

    # Do a little rearranging of objects

    options = []
    imports = []
    databases = []
    datasources = []
    triggers = []
    pages = []
    other = []

    for child in self.rootObject._children:
      if isinstance(child, GFObjects.GFOptions):
        options.append(child)
      elif isinstance(child, GFLibrary.GFImport):
        imports.append(child)
      elif isinstance(child, GFObjects.GFDatabase):
        databases.append(child)
      elif isinstance(child, GFObjects.GFDataSource) or \
        (isinstance(child, GFLibrary.GFImportItem) and
         child._type == 'GFimport-datasource'):
        datasources.append(child)
      elif isinstance(child, GTrigger.GTrigger) or \
        (isinstance(child, GFLibrary.GFImportItem) and
         child._type == 'GFimport-trigger'):
        triggers.append(child)
      elif isinstance(child, GFObjects.GFPage) or \
        (isinstance(child, GFLibrary.GFImportItem) and
         child._type == 'GFimport-page'):
        pages.append(child)
      else:
        other.append(child)


    # Reorder the children so items of same class are grouped
    self.rootObject._children = []
    for child in options:
      self.rootObject._children.append(child)
    for child in imports:
      self.rootObject._children.append(child)
    for child in databases:
      self.rootObject._children.append(child)
    for child in datasources:
      self.rootObject._children.append(child)
    for child in triggers:
      self.rootObject._children.append(child)
    for child in pages:
      self.rootObject._children.append(child)
    for child in other:
      self.rootObject._children.append(child)


  # Hack having to do with UIwxpython import
  def uiEventTrap(self, event):
    pass




