#
# Copyright 2001-2002 Free Software Foundation
#
# This file is part of GNU Enterprise.
#
# GNU Enterprise is free software; you can redistribute it
# and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either
# version 2, or (at your option) any later version.
#
# GNU Enterprise is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied
# warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
# PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public
# License along with program; see the file COPYING. If not,
# write to the Free Software Foundation, Inc., 59 Temple Place
# - Suite 330, Boston, MA 02111-1307, USA.
#
# FILE:
# LayoutEditor.py
#
# DESCRIPTION:
#
# NOTES:


import sys, os
from wxPython.wx import *
from gnue.common import GDebug
from gnue.forms import GFObjects 
from gnue.forms.uidrivers.wx import UIdriver as UIwxpython
from gnue.designer.PopupMenu import PageMenu
from gnue.designer.forms import Incubator
from gnue.designer.TemplateParser import TemplateParser
from gnue.common.events import Event

# My support files
from Utils import *
from LayoutEditorTools import LayoutEditorTools
from GridPane import GridPane
from WidgetHandler import WidgetHandler
from PropertyBar import PropertyBar
from DisplayDropTarget import DisplayDropTarget


class LayoutEditor (wxPanel):
  def __init__(self, instance, parentWindow):
    wxPanel.__init__(self,parentWindow,-1)
    self.frame = instance
    self.panelColor = self.GetBackgroundColour()
    self._instance = instance
    self.rootObject = instance.rootObject
    self._app = instance._app
    self.widgets = {}
    self.widgets.update(UIwxpython.WIDGETS)
    self.toolbar = LayoutEditorTools(self, self.frame)
    del self.widgets['GFPage']

    self.notebook = wxNotebook(self, -1, pos=wxPoint(4, 4), size=wxSize(32,32))
    self.backcolor = wxWHITE
    self.propBar = PropertyBar(self)

    EVT_NOTEBOOK_PAGE_CHANGED(self,self.notebook.GetId(), self.OnPageSelected)
    EVT_SIZE(self, self.OnSize)

    self.page = None
    self.block = None
    self.lastBlock = None
    self.panel = None
    self._notebookMap = {}
    self._pageList = []  # Needed by UIwxpython
    self.positionMappings = {}
    self.widgetList = []

    self.mode = 'move'
    self.__x = 0
    self.__y = 0
    self.__ox = 0
    self.__oy = 0

    instance.rootObject.walk (self.inventoryObject)
    try:
      self._setCurrentPage(self._notebookMap[0])
    except KeyError:
      pass

    self.frame.SetToolBar(self.toolbar)

  def inventoryObject(self, object):
    if object._type == 'GFPage':
      index = self.notebook.GetPageCount()
      self._notebookMap[object] = index
      self._notebookMap[index] = object

      self.workspacescroll = wxScrolledWindow(self.notebook, -1,
         pos=wxPoint(0,0),
         style=wxCLIP_CHILDREN|wxHSCROLL|wxVSCROLL|wxSUNKEN_BORDER)

      self.workspace = wxPanel(self.workspacescroll,-1)
      self.workspace.SetSize((51*20,51*20))

      # TODO: self.workspace needs to have SetScrollbars called with
      # TODO: reasonable values based on size and position of self.panel
      self.workspacescroll.SetScrollbars(20,20,50,50)
      self.workspace.SetBackgroundColour(self.backcolor)

      self.notebook.AddPage(self.workspacescroll,object.name)
      object.__workspace = self.workspace
      self.drawPage(object)


  def onSetCurrentObject (self, object, handler):
    self._currentObject = object
    if object == None:
      for o in self._currentSelection.keys():
        o.setSelected(0)
      return
    if hasattr(object, '_widgetHandler'):
      self.propBar.setCurrentObject(object)
    if handler != "Forms::LayoutEditor":
      self._setCurrentPage(object)


  def _setCurrentPage(self, object):

    page = isinstance(object, GFObjects.GFPage) and object or \
           object.findParentOfType('GFPage')

    if page != None and page != self.page:
      self.page = page
      self.block = None

      self.notebook.SetSelection(self._notebookMap[page])
      try:
        self.workspace = page.__workspace
        self.panel = self.page.__panel
      except AttributeError:
        # This simply means we are setting up
        # our notebook for the first time.
        pass

    block = isinstance(object, GFObjects.GFBlock) and object or \
            object.findParentOfType('GFBlock')

    if block != None:
      self.block = block
    elif self.block is None and self.page is not None:
      # Set block to the initial block
      for child in self.page._children:
        if isinstance(child,GFObjects.GFBlock):
          self.block = child
          break


  def __setCurrentObject(self,object):
    if hasattr(object, '_widgetHandler'):
      self._currentSelection[object._widgetHandler] = 1
      object._widgetHandler.setSelected(1)


  def onCreateObject (self, object, handler):

    self._currentSelection = {}
    if object == None:
      return

    if handler != "Forms::LayoutEditor":
      origpage = self.page

      if object._type != 'GFPage':
        page = object.findParentOfType('GFPage')
        if page:
          self._setCurrentPage(page)
          self.__drawItem(object)

      self.inventoryObject(object)
      self._setCurrentPage(origpage)


  def onModifyObject (self, object, handler, modifications):
    if object == None:
      return

    # TODO: adjust label width to match new length...this shouldn't be here
    if object._type == 'GFLabel':
      mods = []
      for field, value in modifications:
        if field=='text':
          #TODO: this will butcher align=center setups
          if (not hasattr(object,'alignment')) or ( object.alignment == 'left'):
            object.width=len(value)
            mods.append( ('width',len(value)) )
    else:
      mods = modifications

    self.propBar.onModifyObject(object, handler, mods)


    if handler != "Forms::LayoutEditor":
      if object._type == 'GFPage':
        self.notebook.SetPageText(self._notebookMap[object],object.name)
        self.notebook.SetSelection(self.notebook.GetSelection())
      else:
        page = object.findParentOfType('GFPage')
        if page:
          self._setCurrentPage(page)

      if object in (self.rootObject, self.page) or object in self.widgetList:
        self.refreshPage(self.page)


  def onDeleteObject (self, object, handler):

    self._currentObject = None

    if object == None:
      return
    if handler != "Forms::LayoutEditor":
      if object._type == 'GFPage':
        index = self._notebookMap[object]
        for i in range(index+1, len(self._notebookMap.keys())/2):
          self._notebookMap[i-1] = self._notebookMap[i]
        del self._notebookMap[int(len(self._notebookMap.keys())/2)-1]
        del self._notebookMap[object]
        self.notebook.DeletePage(index)

    if hasattr(object, '_widgetHandler') and object._widgetHandler != None:
      try:
        del self._currentSelection[object._widgetHandler]
      except KeyError:
        pass

      for i in range(len(self.widgetList)):
        if self.widgetList[i] == object:
          self.widgetList.pop(i)
          break

      object._widget.cleanup(object)
      object._widget._object = None
      object._widget = None
      object._widgetHandler.Destroy()
      object._widgetHandler = None


  def refreshPage(self, page):
    try:
      page.__panel.Destroy()
      del page.__panel
    except AttributeError:
      pass

    self.drawPage(page)


  def drawPage(self, page):
    self._currentSelection = {}
    self.page = page

    try:
      self.panel = self.page.__panel
    except AttributeError:
      self.panel = GridPane(self, self.workspace, wxPoint(10,12))

      # We are a drop target for Drag-and-Drop
      self.panel.SetDropTarget(DisplayDropTarget(self))


      self.page.__panel = self.panel
      self.panelColor = self.panel.GetBackgroundColour()
      self.panelGridColor = wxColour(order(255,self.panelColor.Red()+16)[0],
                 order(255,self.panelColor.Green()+16)[0],
                 order(255,self.panelColor.Blue()+16)[0])

      UIwxpython.initFont(self.panel)

      maxWidth, maxHeight, maxDescent, maxLeading = [0,0,0,0]

      checkchars = string.letters+string.digits+"-\|" + string.punctuation
      for letter in checkchars:
        width,height,descent,leading = self.panel.GetFullTextExtent(letter)
        maxWidth = maxWidth > width and maxWidth or width
        maxHeight = maxHeight > height and maxHeight or height
        maxDescent = maxDescent > descent and maxDescent or descent
        maxLeading = maxLeading > leading and maxLeading or leading

      self.charWidth = maxWidth+maxLeading
      self.charHeight = maxHeight+maxDescent

      self.borderPercentage = (float(gConfig('borderPercentage')) / 100)
      self.textPercentage = (float(gConfig('textPercentage')) / 100)

      width = self.rootObject.width
      height = self.rootObject.height

      self.widgetWidth = (self.charWidth * self.borderPercentage)
      self.widgetHeight = (self.charHeight * self.borderPercentage) + 3
      self.textWidth = self.charWidth * self.textPercentage
      self.textHeight = self.charHeight * self.textPercentage

      self.gridWidth = self.widgetWidth
      self.gridHeight = self.widgetHeight

      self.menu_sb_space=2 # the extra spaces needed by the menu, toolbar and statusbar

      self.panel.SetClientSize(wxSize(int(width)*int(self.widgetWidth),
                       int(height)*int(self.widgetHeight)))
      self.panel.Refresh()

      page.walk(self.__drawItem)


    EVT_CHAR(self.panel, self.keyTrap)
    EVT_LEFT_DOWN(self.panel, self.OnLeftDown)
    EVT_MOTION(self.panel, self.OnMotion)
    EVT_LEFT_UP(self.panel, self.OnLeftUp)
    EVT_RIGHT_UP(self.panel, self.OnRightUp)


  def __drawItem(self, object):
    if self.widgets.has_key(object._type) and \
       (not hasattr(object, 'hidden') or not object.hidden):
      if hasattr(object, 'name'):
        GDebug.printMesg(7, 'Drawing item %s of type %s' % (object.name, object._type))
      else:
        GDebug.printMesg(7, 'Drawing item of type %s' % (object._type))
      object._widgetHandler = WidgetHandler(self, object)

      cevent = Event('CreateWidget',None,
                   object=object,
                   container=self.panel,
                   textWidth=self.textWidth,
                   textHeight=self.textHeight,
                   widgetWidth=self.widgetWidth,
                   widgetHeight=self.widgetHeight,
                   interface=self,
                   eventHandler=None,
                   initialize=0)

      object._widget = self.widgets[object._type](cevent)
      object._widget._object = object
      object._widgetHandler.initialize(object._widget)
      if not len(self._currentSelection.keys()):
        self._currentSelection = {object._widgetHandler:1}
        object._widgetHandler.setSelected(0)
      self.widgetList.append(object)

  def OnSize(self, event):
    self.propBar.SetSize(wxSize(self.GetClientSize().x, self.propBar.GetSize().y))
    self.propBar.SetPosition(wxPoint(0, self.GetClientSize().y - self.propBar.GetSize().y))
    self.notebook.SetSize(wxSize(self.GetClientSize().x-8, self.GetClientSize().y - 8 - self.propBar.GetSize().y))

  def OnPageSelected(self, event):
    p = self._notebookMap[event.GetSelection()]
    if p != self.page:
      self._instance.onSetCurrentObject(p, "Forms::LayoutEditor")
    self._setCurrentPage(p)


  def OnBlockSelected(self, event):
    b = self.blockList[event.GetSelection()]
#    if p != self.page:
    self._instance.onSetCurrentObject(b, None)


  def OnLeftDown(self, event):
    # Save starting coordinates for cross-selection box
    x, y = event.GetPositionTuple()

    self.panel.SetCursor(wxCROSS_CURSOR)

##    x = int(x / self.gridWidth) * self.gridWidth
##    y = int(y / self.gridHeight) * self.gridHeight

    self.__brush = wxBrush(wxWHITE, style=wxTRANSPARENT)

    self.__x = x
    self.__y = y
    self.__ox = None
    self.__oy = None
    event.Skip()


  # Used by the dragging routines to draw a selection box
  def xorBox(self, x1, y1, x2, y2, x3=None, y3=None):
    dc = wxClientDC(self.panel)
    dc.SetBrush(self.__brush)
    dc.SetPen(wxGREY_PEN)
    dc.SetLogicalFunction(wxXOR)

    if x3 != None:
      dc.DrawRectangle(x1,y1,x3-x1,y3-y1)

    dc.DrawRectangle(x1,y1,x2-x1,y2-y1)


  def OnMotion(self, event):
    # draw cross-selection box
    if event.LeftIsDown():
      x, y = event.GetPositionTuple()
      self.xorBox(self.__x, self.__y, x, y, self.__ox, self.__oy)

      self.__ox = x
      self.__oy = y

    event.Skip()


  def OnLeftUp(self, event):

    for selection in self._currentSelection.keys():
      selection.setSelected(0)

    self._currentSelection = {}

    if self.__ox != None:
      self.xorBox(self.__x, self.__y, self.__ox, self.__oy)
      self.__ox = None
      self.__oy = None

    x, y = event.GetPositionTuple()

    # Set x1,y1 to be smaller coordinates, and x2,y2 to be larger
    x1, x2 = order(int(self.__x / self.gridWidth), int(x / self.gridWidth))
    y1, y2 = order(int(self.__y / self.gridHeight),int(y / self.gridHeight))


    self.panel.SetCursor(wxSTANDARD_CURSOR)
    wxSetCursor(wxSTANDARD_CURSOR)


    areaSelected = (x1 <> x2 or y1 <> y2)

    event.Skip()

    if self.mode == 'move':
      if areaSelected:
        # We are selecting an area
        self.page.walk(self.selectWidgetInArea, x1, y1, x2, y2)

    elif self.mode == 'positioning':

       if areaSelected:
         width = x2 - x1 + 1
         height = y2 - y1 + 1
       else:
         width = None
         height = None

       PrepositioningTimer(self.endPrePositioningTemplate,x1,y1,width,height).Start(100,1)




  def selectWidgetInArea(self, object, x1, y1, x2, y2):
    try:
      if x1 <= object.x <= x2 and \
         y1 <= object.y <= y2:
        self._currentSelection[object._widgetHandler] = 1
        object._widgetHandler.setSelected(1)
    except AttributeError:
      # Not all child objects have coordinates (x,y)
      pass


  def OnRightUp(self, event):
    self.toolbar.resetTool(self.mode)
    self.mode = 'move'

    x, y = event.GetPositionTuple()

    x = int(x / self.gridWidth)
    y = int(y / self.gridHeight)

    menu = wxMenu('Layout Editor')
    form = self._currentObject._type == 'GFForm' and self._currentObject or \
       self._currentObject.findParentOfType('GFForm')
    page = self._currentObject._type == 'GFPage' and self._currentObject or \
       self._currentObject.findParentOfType('GFPage')
    block = self._currentObject._type == 'GFBlock' and self._currentObject or \
       self._currentObject.findParentOfType('GFBlock')

    menu.AppendMenu(wxNewId(), 'Form', PageMenu(self._instance, form, x, y))
    if page:
      menu.AppendMenu(wxNewId(), page.name, PageMenu(self._instance, page, x, y))
    if block:
      menu.AppendMenu(wxNewId(), block.name, PageMenu(self._instance, block, x, y))

    self.panel.PopupMenu(menu, event.GetPosition())


  def clearSelections(self):
    for widget in self._currentSelection.keys():
      widget.setSelected(0)
    self._currentSelection = {}


  def keyTrap(self, event):
    if event.KeyCode() in (WXK_LEFT, WXK_RIGHT, WXK_UP, WXK_DOWN) and \
      len(self._currentSelection.keys()):
      if event.AltDown(): # or event.MetaDown():
        if event.KeyCode() == WXK_LEFT:
          for widget in self._currentSelection.keys():
            widget.relativeResize(-1,0)
        if event.KeyCode() == WXK_RIGHT:
          for widget in self._currentSelection.keys():
            widget.relativeResize(1,0)
        if event.KeyCode() == WXK_UP:
          for widget in self._currentSelection.keys():
            widget.relativeResize(0,-1)
        if event.KeyCode() == WXK_DOWN:
          for widget in self._currentSelection.keys():
            widget.relativeResize(0,1)
      else:
        if event.KeyCode() == WXK_LEFT:
          for widget in self._currentSelection.keys():
            widget.relativeMove(-1,0)
        if event.KeyCode() == WXK_RIGHT:
          for widget in self._currentSelection.keys():
            widget.relativeMove(1,0)
        if event.KeyCode() == WXK_UP:
          for widget in self._currentSelection.keys():
            widget.relativeMove(0,-1)
        if event.KeyCode() == WXK_DOWN:
          for widget in self._currentSelection.keys():
            widget.relativeMove(0,1)


  def startPrePositioningTemplate(self, template):
    self.mode = 'positioning'
    ##self.panel.SetCursor(wxCROSS_CURSOR)
    wxSetCursor(wxCROSS_CURSOR)
    self.__template = template


  def endPrePositioningTemplate(self, x, y, width=None, height=None):
    TemplateParser(self._instance, self._instance.rootObject,
          self._instance, self.__template, currentObject=self.block or self.page).run(x=x, y=y,
                                               width=width,
                                               height=height)
    self._instance.dispatchEvent(Event('EndWizard', template=self.__template))
    self.mode = 'move'



class PrepositioningTimer(wxTimer):
    def __init__(self, method, *args, **params):
      self.__method = method
      self.__args = args
      self.__params = params
      wxTimer.__init__(self)

    def Notify(self):
      self.__method(*self.__args, **self.__params)


