#
# This file is part of GNU Enterprise.
#
# GNU Enterprise is free software; you can redistribute it
# and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either
# version 2, or (at your option) any later version.
#
# GNU Enterprise is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied
# warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
# PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public
# License along with program; see the file COPYING. If not,
# write to the Free Software Foundation, Inc., 59 Temple Place
# - Suite 330, Boston, MA 02111-1307, USA.
#
# Copyright 2001-2002 Free Software Foundation
#
# FILE:
# LayoutEditor.py
#
# DESCRIPTION:
#
# NOTES:

import os, string
from wxPython.wx import *
from gnue.common import GDebug, GConfig

from gnue.common.events import Event, EventAware

iconlocation = GConfig.getInstalledBase('designer_images','common_images') + '/'


ToolbarMapping = 'File|New|Form,tb_new.png;'   \
               + 'File|Save,tb_save.png;'   \
               + ';'   \
               + 'Edit|Insert|Label,deslay_label.png;' \
               + 'Edit|Insert|Unbound Entry,deslay_entrytext.png;'   \
               + 'Edit|Insert|Drop Down Entry,deslay_entrydrop.png;' \
               + 'Edit|Insert|Button,deslay_button.png;' \
               + 'Edit|Insert|Box,deslay_box.png'

#               + 'File|Open,tb_open.png;'   \


class LayoutEditorTools(wxToolBar, EventAware):

  def __init__(self, editor, frame):
    self.editor = editor
    wxToolBar.__init__(self, frame, -1)#, style=wxTB_DOCKABLE)
    EventAware.__init__(self, editor._instance)
    self.registerEventListeners( {'BeginWizard' : self.onBeginWizard,
                                  'EndWizard' : self.onEndWizard,
                                  'MakeDirty' : self.onDirtyForm,
                                  'MakeClean' : self.onCleanForm, } )


    self.mapping = {}
    self.SetToolBitmapSize(wxSize(32,32))

    for type in string.split(ToolbarMapping,';'):
      if not len(type):
        self.AddSeparator()
      else:
        handle, image = string.split(type,',',1)
        help = self.editor._instance._menubar.getEventHelp(handle)

        if not help:
          print "Warning: No event tied to %s" % handle
          help = ""

        id = wxNewId()
        self.mapping[id] = handle
        self.mapping[handle] = id

        self.AddTool(id,
          wxImage(os.path.join(iconlocation,image),
                  wxBITMAP_TYPE_PNG).ConvertToBitmap(),
          isToggle=1,
          shortHelpString=help,
          longHelpString=help)

        self.SetToolClientData(id, handle)

        EVT_TOOL(self, id, self.OnButtonSelected)

      wxToolBar.Realize(self)

  def resetTool(self, mode):
    if self.mapping.has_key(mode):
      self.ToggleTool(self.mapping[mode],0)


  def OnButtonSelected(self, event):
    self.ToggleTool(event.GetId(),0)
    handle = self.mapping[event.GetId()]
    self.editor._instance._menubar.simulateMenuEvent(handle)

##    if self.editor.mode == handle:
##      self.editor.mode = 'move'
##    else:
##      self.resetTool(self.editor.mode)
##      self.editor.mode = handle

##    self.editor.frame.SetStatusText('Click on the gridded Layout '
##           'Editor to create a new %s.' % descr)

  def onBeginWizard(self, event):
    try:
      self.ToggleTool(self.mapping[event.template['Location']],1)
    except KeyError:
      pass

  def onEndWizard(self, event):
    try:
      self.ToggleTool(self.mapping[event.template['Location']],0)
    except KeyError:
      pass

  def onDirtyForm(self, event):
    try:
      self.EnableTool(self.mapping['File|Save'],1)
    except KeyError:
      pass


  def onCleanForm(self, event):
    try:
      self.EnableTool(self.mapping['File|Save'],0)
    except KeyError:
      pass
