#
# Copyright 2001-2002 Free Software Foundation
#
# This file is part of GNU Enterprise.
#
# GNU Enterprise is free software; you can redistribute it
# and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either
# version 2, or (at your option) any later version.
#
# GNU Enterprise is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied
# warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
# PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public
# License along with program; see the file COPYING. If not,
# write to the Free Software Foundation, Inc., 59 Temple Place
# - Suite 330, Boston, MA 02111-1307, USA.
#
# FILE:
# WidgetHandler.py
#
# DESCRIPTION:
#
# NOTES:

from wxPython.wx import *
import sys, string
from Utils import *
from gnue.common import GDebug
import cPickle

# LayoutEditor support files
from WidgetHighlighter import WidgetHighlighter
from PopupEditor import PopupEditor

#
#
#
class WidgetHandler(wxEvtHandler):
  def __init__(self, instance, object):
    wxEvtHandler.__init__(self)
    self.instance = instance
    self.object = object
    self.hasFocus = 0
    self.highlightBox = WidgetHighlighter(self.instance.panel)
    self._iMoved = 0
    self._alreadySelected = 0
    self._popup_editor = None

  def initialize(self, widget):
    self.widget = widget.widgets[0]
    self.mainWidget = widget
    self.selected = 0

    self.recalcBoundaries()

    for widget in self.mainWidget.widgets:
      widget.SetCursor (wxSTANDARD_CURSOR)
      if isinstance(widget, wxWindow):
        widget.Enable(0)
      self.setAllChildren(EVT_KEY_UP, widget, self.instance.keyTrap)
      self.setAllChildren(EVT_SET_FOCUS, widget, self.focusGainedTrap)
      self.setAllChildren(EVT_KILL_FOCUS, widget, self.focusLostTrap)
      self.setAllChildren(EVT_LEFT_UP, widget, self.OnLeftUp)
      self.setAllChildren(EVT_LEFT_DOWN, widget, self.OnLeftDown)
      self.setAllChildren(EVT_RIGHT_DOWN, widget, self.OnRightDown)
      self.setAllChildren(EVT_RIGHT_UP, widget, self.OnRightUp)
      self.setAllChildren(EVT_MOTION, widget, self.OnMotion)
      self.setAllChildren(EVT_LEFT_DCLICK, widget, self.OnEditProperties)
    self.setAllChildren(EVT_KEY_UP, self.highlightBox, self.instance.keyTrap)
    self.setAllChildren(EVT_SET_FOCUS, self.highlightBox, self.focusGainedTrap)
    self.setAllChildren(EVT_KILL_FOCUS, self.highlightBox, self.focusLostTrap)
    self.setAllChildren(EVT_LEFT_UP, self.highlightBox, self.OnLeftUp)
    self.setAllChildren(EVT_LEFT_DOWN, self.highlightBox, self.OnLeftDown)
    self.setAllChildren(EVT_RIGHT_UP, self.highlightBox, self.OnRightUp)
    self.setAllChildren(EVT_MOTION, self.highlightBox, self.OnMotion)

    self.widget.Refresh()

  def recalcBoundaries(self):
    x1,y1,x2,y2 = (9999999,9999999,0,0)

    for widget in self.mainWidget.widgets:
      tx, ty = widget.GetPositionTuple()
      tw, th = widget.GetSizeTuple()
      self.instance.positionMappings[widget] = [tx,ty,tx+tw-1,ty+tw-1,self]
      if tx < x1:
        x1 = tx
      if ty < y1:
        y1 = ty
      if tw + tx > x2:
        x2 = tw + tx
      if th + ty > y2:
        y2 = th + ty

    self.highlightBox.setBoundary(x1-2,y1-2,x2+2,y2+2)
    for widget in self.mainWidget.widgets:
       widget.Refresh()


  def setSelected(self, selected):
    self.selected = selected
    self.highlightBox.setSelected(selected)


  def setAllChildren(self, event, widget, trap):
    try:
      event(widget, trap)
    except:
      pass
    for child in widget.GetChildren():
      self.setAllChildren(event, child, trap)

  def focusGainedTrap(self, event):
    self.instance.panel.SetFocus()
    pass
#    event.Skip()

  def focusLostTrap(self, event):
    event.Skip()


  def OnLeftDown(self, event):
    self.mouseStartX, self.mouseStartY = event.GetPositionTuple()
    self._iMoved = 0

    if not self.instance._currentSelection.has_key(self):
      self.instance._currentSelection[self] = 1
      self.instance._instance.onSetCurrentObject(self.object,"Forms::LayoutEditor")
      self.setSelected(1)
      self._alreadySelected = 0
    else:
      self._alreadySelected = 1

    event.Skip()


  def beginDragDrop(self, event):

    # Drag and drop TODO: This needs changed to pull all selected widgets

    object = self.object

    x,y = event.GetPositionTuple()

    data = [ { "Type" : "selectedWidgets",
               "Attributes": {
                   'startingX': object.x + int(x/self.instance.gridWidth),
                   'startingY': object.y + int(y/self.instance.gridHeight),
                   }
               }]
#        "name": dname,
#        "table": object.name,
#        "database": self.connectionName } }
#
#
    do = wxCustomDataObject(wxCustomDataFormat("GNUeDesVisualElement"))
    do.SetData(cPickle.dumps(data,1))

##    icon = FeedbackBitmap(10,10).icon()
    dropSource = MoveDropSource(self, self.instance)
    dropSource.SetData(do)
    result = dropSource.DoDragDrop(true)
    event.Skip()


  def OnLeftUp(self, event):
    if self._iMoved:
      self._iMoved = 0
    elif not event.ShiftDown():
      # Select only 1 widget
      for selection in self.instance._currentSelection.keys():
        selection.setSelected(0)
      self.instance._currentSelection = {self:1}
      self.setSelected(1)
    elif self._alreadySelected:
      # Remove widget from selection hash
      del self.instance._currentSelection[self]
      self.setSelected(0)

    self._alreadySelected = 0
    event.Skip()


  def OnMotion(self, event):

    if not self._iMoved and event.LeftIsDown():
      self._iMoved = 1
      self.beginDragDrop(event)

    event.Skip()


  def OnRightDown(self, event):
    event.Skip()


  def OnRightUp(self, event):
    pt = event.GetPositionTuple()
    self.instance._instance.onSetCurrentObject(self.object, None)
    wxPyTypeCast(event.GetEventObject(),'wxWindow') \
       .PopupMenu(self.object._popupMenu, pt)
    event.Skip()


  def _endPopupEditor(self, success, value=""):
    if success:
      self.object.__dict__[self.__popupAttr] = value
      self.instance._instance.onModifyObject(self.object,
            "Forms::LayoutEditor", ((self.__popupAttr, value),))
      self._popup_editor.Destroy()
      self._popup_editor = None
      self.widget.SetLabel(value or "<Unset>")
      if self.object._type == 'GFLabel':
        width = len(value) or 10
        self.widget.SetSize((width * self.instance.gridWidth,
                            self.widget.GetSize().height))
        self.recalcBoundaries()
        self.setSelected(self.selected)


  def OnEditProperties(self, event):

    if self.object._type in ('GFBox','GFLabel'):
      # Create a popup text editor for the labels
      if self.object._type == 'GFBox':
        self.__popupAttr = 'label'
        x = self.widget.GetPosition().x + (self.instance.gridWidth/2)
        width = (self.object.width - 2) * self.instance.gridWidth
      else:
        self.__popupAttr = 'text'
        x = self.widget.GetPosition().x
        width = self.widget.GetSize().width + self.instance.gridWidth

      self.instance.clearSelections()
      self._popup_editor = PopupEditor(self.instance.workspace,
          self._endPopupEditor,
          x + self.instance.panel.GetPosition().x,
          self.widget.GetPosition().y + self.instance.panel.GetPosition().y,
          width,
          self.instance.gridHeight,
          self.object.__dict__[self.__popupAttr])

      self._popup_editor.SetFocus()

    else:
      # Signal the Properties dialog
      self.instance._instance.propertyEditorWindow.Show(1)
      self.instance._instance.propertyEditorWindow.Raise()
      self.instance._instance.propertyEditorWindow.SetFocus()

  #
  # Resize the current widget.  dx and dy are incremental
  # resize units. (e.g, (-1,0) means make the widget one
  # unit smaller than it currently is.
  #
  def relativeResize(self, dx, dy):
    if not (dx or dy):
      return 0

    nx = dx * self.instance.gridWidth
    ny = dy * self.instance.gridHeight

    if (self.object.width + dx < 0 or self.object.height + dy < 0):
      return 0
    else:
      for widget in self.mainWidget.widgets:
        x,y = widget.GetSizeTuple()
        widget.SetSize(wxSize(x + nx,y + ny))
      changes = []
      if dx:
        self.object.width = self.object.width + dx
        changes.append(['width',self.object.width])
      if dy:
        self.object.height = self.object.height + dy
        changes.append(['height',self.object.height])
      self.instance._instance.onModifyObject(self.object, "Forms::LayoutEditor", changes)
      self.recalcBoundaries()


  #
  # Move the current widget. dx and dy are increments.
  # (e.g., (-1,-2) means move the object up one unit
  # and left two units.
  #
  def relativeMove(self, dx, dy):
    if not (dx or dy):
      return 0

    nx = dx * self.instance.gridWidth
    ny = dy * self.instance.gridHeight

    if (self.object.x + dx < 0 or self.object.y + dy < 0):
      return 0
    else:
      for widget in self.mainWidget.widgets:
        x,y = widget.GetPositionTuple()
        widget.SetPosition(wxPoint(x + nx,y + ny))
      changes = []
      if dx:
        self.object.x = self.object.x + dx
        changes.append(['x',self.object.x])
      if dy:
        self.object.y = self.object.y + dy
        changes.append(['y',self.object.y])
      self.instance._instance.onModifyObject(self.object,
           "Forms::LayoutEditor", changes)
      self.recalcBoundaries()

  #
  # Recursively destroy this widget set
  #
  def Destroy(self):
    
    # TODO: Why the fsck can't I Destroy() this thing!?!??!?
    self.widget.Show(0)

    self.instance = None
    self.object = None
    self.highlightBox.Destroy()
    self.highlightBox = None
    self._popup_editor = None
    self.widget = None
    self.mainWidget = None
