#
# This file is part of GNU Enterprise.
#
# GNU Enterprise is free software; you can redistribute it
# and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either
# version 2, or (at your option) any later version.
#
# GNU Enterprise is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied
# warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
# PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public
# License along with program; see the file COPYING. If not,
# write to the Free Software Foundation, Inc., 59 Temple Place
# - Suite 330, Boston, MA 02111-1307, USA.
#
# Copyright 2001-2002 Free Software Foundation
#
# FILE:
# reports/Instance.py
#
# DESCRIPTION:
# Creates a "report" instance
#
# NOTES:
#

from gnue.designer.reports import properties as reportsProperties
from gnue.designer.Instance import BaseInstance
import Incubator
import wizards

# Tool support...
from LayoutEditor import LayoutEditor
from TreeView import TreeView
from gnue.designer.PropertyEditor import PropertyEditor
from gnue.designer.DataSourceEditor import DataSourceEditor
from gnue.designer.SchemaViewer import SchemaViewer
from gnue.designer.TriggerEditor import TriggerEditor

from gnue.reports import GRReport, GRParser, GRLayout


class Instance(BaseInstance):

  def __init__(self, app, *args, **params):
    self.incubator = Incubator
    self.properties = reportsProperties
    
    #self._tableMappings = {}
    BaseInstance.__init__(self, app, *args, **params)


  def loadBuffer(self, buffer):
    report = GRParser.loadFile (buffer, self, initialize=0)
    report.name = 'report'
    return report


  def loadEmpty(self):
    report = GRReport.GRReport()
    report.title = "Untitled report"
    report.name = 'report'
    return report


  def inventoryObject(self, object):
	pass

  def createVisualEditor(self):
    return LayoutEditor(self, self)


  def createTools(self):
    self.addTool(_('propertyEditor'), _('Property Inspector'), PropertyEditor)
## TODO: Causing a segfault with Wizards!!!!
    self.addTool(_('treeEditor'), _('Object Navigator'), TreeView)
    self.addTool('triggerEditor', _('Event Editor'), TriggerEditor)

    self.addTool(_('schemaViewer'), _('Schema Navigator'), SchemaViewer)

    self.loadWizards(wizards)


  def preSave(self):
    pass

  # Hack having to do with UIwxpython import
  def uiEventTrap(self, event):
    pass





