#
# This file is part of GNU Enterprise.
#
# GNU Enterprise is free software; you can redistribute it
# and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either
# version 2, or (at your option) any later version.
#
# GNU Enterprise is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied
# warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
# PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public
# License along with program; see the file COPYING. If not,
# write to the Free Software Foundation, Inc., 59 Temple Place
# - Suite 330, Boston, MA 02111-1307, USA.
#
# Copyright 2001-2002 Free Software Foundation
#
# FILE:
# schema/Instance.py
#
# DESCRIPTION:
# Creates a "schema" instance
#
# NOTES:
#

from gnue.designer.schema import properties as schemaProperties
from gnue.designer.Instance import BaseInstance
from gnue.common.schema import GSParser, Objects
import Incubator
import wizards

# Tool support...
from LayoutEditor import LayoutEditor
from TreeView import TreeView
from gnue.designer.PropertyEditor import PropertyEditor
from gnue.designer.DataSourceEditor import DataSourceEditor
from gnue.designer.SchemaViewer import SchemaViewer
from gnue.designer.schema.VisualEditor import VisualEditor



class Instance(BaseInstance):

  def __init__(self, app, *args, **params):
    self.incubator = Incubator
    self.properties = schemaProperties
    
    self._tableMappings = {}
    BaseInstance.__init__(self, app, *args, **params)


  def loadBuffer(self, buffer):
    schema = GSParser.loadSchema (buffer, self, initialize=0)
    schema.name = 'schema'
    return schema


  def loadEmpty(self):
    schema = Objects.GSSchema()
    schema.title = "Untitled schema"
    schema.name = 'schema'
    return schema


  def inventoryObject(self, object):
    if isinstance(object, Objects.GSTables):
      self.tables = object

    if isinstance(object, Objects.GSTable):
      self._tableMappings[object.name] = object


  def createVisualEditor(self):
    return LayoutEditor(self, self)


  def createTools(self):
    self.addTool(_('treeEditor'), _('Object Navigator'), TreeView)
    self.addTool(_('propertyEditor'), _('Property Inspector'), PropertyEditor)
    self.addTool(_('schemaViewer'), _('Schema Navigator'), SchemaViewer)
    self.addTool(_('visualEditor'), _('Visual Editor'), VisualEditor)
    self.loadWizards(wizards)


  def preSave(self):
    pass

  # Hack having to do with UIwxpython import
  def uiEventTrap(self, event):
    pass




